﻿using System;
using System.Text;

using NATS.Client;

namespace TestCommon
{
    /// <summary>
    /// 송신기
    /// </summary>
    public class Sender : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 메시지 서버 URL
        /// </summary>
        private string messageServerURL = null;

        /// <summary>
        /// 메시지 주제
        /// </summary>
        private string messageSubject = null;

        /// <summary>
        /// 연결
        /// </summary>
        private IConnection connection = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - Sender()

        /// <summary>
        /// 생성자
        /// </summary>
        public Sender()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 초기화 하기 - Initialize(messageServerURL, messageSubject)

        /// <summary>
        /// 초기화 하기
        /// </summary>
        /// <param name="messageServerURL">메시지 서버 URL</param>
        /// <param name="messageSubject">메시지 주제</param>
        public void Initialize(string messageServerURL, string messageSubject)
        {
            this.messageServerURL = messageServerURL;
            this.messageSubject   = messageSubject;

            Options options = ConnectionFactory.GetDefaultOptions();

            options.Url = this.messageServerURL;

            this.connection = new ConnectionFactory().CreateConnection(options);
        }

        #endregion
        #region 메시지 송신하기 - SendMessage(source)

        /// <summary>
        /// 메시지 송신하기
        /// </summary>
        /// <param name="source">소스 문자열</param>
        public void SendMessage(string source)
        {
            if(this.connection != null)
            {
                byte[] sourceByteArray = Encoding.UTF8.GetBytes(source);

                this.connection.Publish(this.messageSubject, sourceByteArray);

                this.connection.Flush();
            }
        }

        #endregion
        #region 메시지 요청하기 - RequestMessage(source, timeout)

        /// <summary>
        /// 메시지 요청하기
        /// </summary>
        /// <param name="source">소스 문자열</param>
        /// <param name="timeout">타임아웃</param>
        /// <returns>결과 문자열</returns>
        public string RequestMessage(string source, int timeout)
        {
            if(this.connection != null)
            {
                byte[] sourceByteArray = Encoding.UTF8.GetBytes(source);

                Msg replyMessage = this.connection.Request(this.messageSubject, sourceByteArray, timeout);

                string reply = Encoding.UTF8.GetString(replyMessage.Data);

                return reply;
            }
            else
            {
                return null;
            }
        }

        #endregion

        #region (IDisposable) 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            if(this.connection != null)
            {
                this.connection.Dispose();
            }
        }

        #endregion
    }
}